<?php 

namespace Wutime\AdBlock\Cron;

class CreateJavaScript
{

	static $dupe_check = []; // keeps a record of folders used
	static $enabled = false; // set during setup()
	static $path = '';  // set during setup()
	static $oldAge = 60*60*2;  // number of seconds to consider files old (default 2 hours)
	static $folder = 'js';
	static $separator = '/';
	static $alwaysKeepAtleastOldCopies = 2;
	static $default_js_key = 'elaoianb32k';
	static $default_js_detect_file = 'wutime_detect.min';
	static $default_filename = 'ads.min.js';
	static $default_foldername = 'wutime_adblock';

	/**
	*  Primary function for the cron
	*/
	public static function run()
	{
		$options = \XF::options();
		self::setPath();
		self::$enabled = ($options->wutime_adblock_auto_randomize && $options->wutime_adblock_enable);
		self::$oldAge = $options->wutime_adblock_delete_files_older_than_x_hours * 60 * 60;

		if (self::$enabled) {
			self::createJavaScript();
		}
		self::deleteOldJavaScriptFiles();
	}


	/** 
	* uninstall
	*/
	public static function uninstall()
	{
		self::deleteAllJavaScriptFiles();
	}

	/**
	*  Primary function for the cron
	*/
	public static function install()
	{
		self::setPath();
		self::createJavaScript();
	}


	/**
	*  Setup path
	*/
	public static function setPath() {
		self::$path = \XF::getRootDirectory() . self::$separator . self::$folder;
	}

	/**
	*  Create JavaScript files
	*/
	static function createJavaScript()
	{
		// this only exists after initial run
		$oldJavascript = \XF::em()->findOne('Wutime\AdBlock:JavaScript', ['status' =>  'New']);
        // load the old keys for  dupe checking
		if($oldJavascript) {
			self::$dupe_check[] = $oldJavascript->js_key;
			self::$dupe_check[] = $oldJavascript->js_detect_file;
			self::$dupe_check[] = $oldJavascript->foldername;
		}
		// create new JavaScript
        $newJavascript = \XF::em()->create('Wutime\AdBlock:JavaScript');
        $newJavascript->status 			= 'New';

        if (!$newJavascript->filename = self::randomFile()) {
        	$newJavascript->filename = self::getFilenameFallback();
        }
        

        // make sure our new folder doesn't already exist
        // can't risk deleting or using existing directories
        do {
        	$newJavascript->foldername = self::randomString(rand(4,12));
        } while (file_exists(self::getFilename($newJavascript->foldername)));

        $newJavascript->js_key 			= self::randomString(rand(10,20));
        $newJavascript->js_detect_file 	= self::randomString(rand(10,20));
 		$newJavascript->path 			= self::getFilename($newJavascript->foldername);
        $newJavascript->save();

        // create file pointers
        $js_file = self::getFilename($newJavascript->foldername,$newJavascript->filename);
        $js_detect_file = self::getFilename($newJavascript->foldername,$newJavascript->js_detect_file).'.js';

        // create new files
		if (\XF\Util\File::isWritable($js_file) && \XF\Util\File::isWritable($js_detect_file))  {
			\XF\Util\File::writeFile($js_file, self::getJavascript($newJavascript->js_key));
			\XF\Util\File::writeFile($js_detect_file, self::getJavaScriptDetect($newJavascript->js_key));
		} else {
			\XF::logError('[Wutime\AdBlock] /js/ folder is not writable.');

	        $newJavascript->js_key 			= self::$default_js_key;
	        $newJavascript->js_detect_file 	= self::$default_js_detect_file;
	        $newJavascript->filename 		= self::$default_filename;
	        $newJavascript->foldername 		= self::$default_foldername;
	 		$newJavascript->path 			= self::getFilename($newJavascript->foldername);
	        $newJavascript->save();
		}

        // update and delete old javascript
		if($oldJavascript) {
			// make old
			$oldJavascript->status = 'Old';
			// save changes
			$oldJavascript->save();
		}
		return;
	}

	/** 
	* Deletes all old files (older than X hours) (always keep atleast)
	*/
	public static function deleteOldJavaScriptFiles()
	{
		$oldJavascripts = \XF::finder('Wutime\AdBlock:JavaScript')
                                ->where('status', 'Old')
                                ->where('created','<',(\XF::$time-self::$oldAge))
                                ->order('created', 'ASC')
                                ->fetch();
        if ($oldJavascripts) {
        	if (count($oldJavascripts) >= self::$alwaysKeepAtleastOldCopies) {
        		$cnt = count($oldJavascripts)-self::$alwaysKeepAtleastOldCopies+1;
				foreach ($oldJavascripts as $JavaScriptEntity)  {
					$JavaScriptEntity->delete();
					$cnt--;
					if ($cnt==0) {
						break;
					}
				}	
        	}
        }      
        return;            
	}

	protected static function getJavaScript($js_key) {
		return "var e=document.createElement('div');e.id=window.btoa(window.$js_key);e.style.display='none';document.body.appendChild(e);";
	}

	protected static function getJavaScriptDetect($js_key) {
		return "var $js_key = btoa(window.location.hostname + new Date().valueOf()).replace(/[^a-zA-Z0-9]/g, '');";
	}

	protected static function getFilename($folder, $filename=null) {
		$path =self::$path.self::$separator.$folder;
		if ($filename) $path .= self::$separator.$filename;
		return $path;
	}

	protected static function randomString($length)
	{
		$start_characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_';
	    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_';
	    $charactersLength = strlen($characters);
	    do {
	    	$randomString = $start_characters[rand(0, strlen($start_characters) - 1)];
		    for ($i = 1; $i < $length; $i++) {
		        $randomString .= $characters[rand(0, $charactersLength - 1)];
		    }
		} while (in_array($randomString,self::$dupe_check));
		self::$dupe_check[] = $randomString;
	    return $randomString;
	}

	protected static function randomFile()
	{
		$options = \XF::options();
		$filenames = self::getFilenameFallback();

		if (isset($options->wutime_adblock_recent_easylist)) {
			if ($options->wutime_adblock_recent_easylist) {
				if ($myfilenames = self::getDownloadedListFilenames()) {
					$filenames = $myfilenames;
				} else {
					\XF::logError('[Wutime\AdBlock] getDownloadedListFilenames() failed to download the selected Easy List.');
				}
			}
		} 

		// get random filename
		$filename = $filenames[rand(0,count($filenames)-1)];

		return $filename;
	}

	protected static function deleteDirectory($folder)
	{
		\XF\Util\File::deleteDirectory($folder);
	}

	/** 
	* Deletes all old files (new and old) (this is called directly on uninstall())
	*/
	public static function deleteAllJavaScriptFiles()
	{
		while ($oldJavascript = \XF::em()->findOne('Wutime\AdBlock:JavaScript', ['status' => ['New','Old']])) {
			if ($oldJavascript)  {
				$oldJavascript->delete();
			}
		}
	}


	private static function getFileContentsFromUrl($url) {

		$file = false;

	    if (function_exists('curl_version')) {

	        $curl = curl_init();
	        curl_setopt($curl, CURLOPT_URL, $url);
	        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
	        curl_setopt($curl, CURLOPT_FAILONERROR, true);
	        $file = curl_exec($curl);

	        if (curl_errno($curl)) {
			    $error_msg = curl_error($curl);
			    \XF::logError('[Wutime\AdBlock] CURL ERROR: ' . $error_msg);
			   	$file = false;
			}
	        curl_close($curl);

	    } elseif (file_get_contents(__FILE__) && ini_get('allow_url_fopen')) {
			try {
			    $file = file_get_contents($url);

			    if ($file === false) {
			        \XF::logError('[Wutime\AdBlock] allow_url_fopen is failing.');
			    }
			} catch (Exception $e) {
			    \XF::logError('[Wutime\AdBlock] Caught exception: ' . $e->getMessage());
			}

	    } else {
	    	\XF::logError('[Wutime\AdBlock] No available PHP method to download URLs.');     
	    }	

	    return $file;
	}

	private static function getDownloadedListFilenames() {

		$options = \XF::options();

		$default_url = 'https://easylist.to/easylist/easylist.txt';
		$max_records = 100;
		$choose_records = 10;
		$filenames = array();
		$hash_length = 3;
		$print_finds = false;
		$print_results = false;

		// get randmoness (for "my-", "this-")
		$randomletters = substr(str_shuffle("abcdefghijklmnopqrstuvwxyz"), 0, $hash_length);

		// look for download url and get javascript identifiers
		if (!$file = self::getFileContentsFromUrl($options->wutime_adblock_recent_easylist_url) ) {
			if ($options->wutime_adblock_recent_easylist_url != $default_url) {
				\XF::logError('[Wutime\AdBlock] CUSTOM remote easy list ('.$options->wutime_adblock_recent_easylist_url.') could not be downloaded (falling back to DEFAULT).');
				if (!$file = self::getFileContentsFromUrl($default_url) ) {
					\XF::logError('[Wutime\AdBlock] DEFAULT remote easy list ('.$default_url.') could not be downloaded.');
					return false;
				}
			} else {
				\XF::logError('[Wutime\AdBlock] DEFAULT remote easy list ('.$default_url.') could not be downloaded.');
				return false;
			}
		}

		try {
			preg_match_all('#^([a-zA-Z0-9-\/]{1}([a-zA-Z0-9-\.])*[a-zA-Z0-9-\.](.js|\.|\-|\_)+)$#m', $file, $matches, PREG_SET_ORDER);

			/* get matches count */
			if ($max_records > count($matches)) {
				$max_records = count($matches);
			}

			if ($max_records < 1) {
				\XF::logError('[Wutime\AdBlock] Could not preg_match at least 1 value (falling back to FALLBACK).');
				return false;
			}

			$randomized = array_rand($matches, $max_records);

			foreach ($randomized as $key => $val) {
				$filenames[] = $matches[$val][0]; 
			}

			shuffle($filenames);

			$patterns[] = '#^/#m'; // remove first /slash
			$patterns[] = '#^-#m'; // replace first hyphem
			$patterns[] = '#\.$#m'; // catch ending in .
			$patterns[] = '#-$#m'; // catch ending in -
			$patterns[] = '#_$#m'; // catch ending in _

			$replacements[] = '';
			$replacements[] = $randomletters.'-';
			$replacements[] = '.js';
			$replacements[] = '-'.$randomletters.'.js';
			$replacements[] = '_'.$randomletters.'.js';

			$filenames = preg_replace($patterns, $replacements, $filenames);

			$cnt = 0;

			// remove any with a / in it
			for ($cnt = 0; $cnt < count($filenames); $cnt++) {
				if ( substr_count ($filenames[$cnt], '/') > 0 ) {
					unset($filenames[$cnt]);
				}
			}

			$selected = array_rand($filenames, $choose_records);

			foreach ($selected as $key => $val) {
				$chosen[] = $filenames[$val];
			}
		} catch (Exception $e) {
			\XF::logError('[Wutime\AdBlock] Caught exception: ' . $e->getMessage());
			return false;
		}

		return $chosen;

	}


	private static function getFilenameFallback() {
		return ['feedads.js',
				'advertsquare.js',
				'google-adverts-.js',
				'carouselads.js',
				'img.ads.js',
				'remove-ads.js',
				'adsjs.js',
				'ads.js',
				'idleAds.js',
				'ads-request.js',
				'imads.js',
				'adsnew.js',
				'adshow_.js',
				'ads-top.js',
				'adsdelivery.js',
				'loadadsmain.js',
				'page-ads.js'];
	}
}