<?php

namespace Wutime\AdBlock;

class Listener
{   
    public static function templaterTemplatePreRenderPublicScriptDetection(\XF\Template\Templater $templater, &$type, &$template, array &$params) {
        //\XF::logError('templaterTemplatePreRenderPublicScriptDetection');

        /**
        * Database JavaScript Elements
        **/
        if (\XF::options()->wutime_adblock_auto_randomize) {
            // Get 'New' JavaScript
            $Javascript = \XF::em()->findOne('Wutime\AdBlock:JavaScript', ['status' =>  'New']);
            if ($Javascript) {
                $params['wutime_adblock_db_foldername'] = $Javascript->foldername;
                $params['wutime_adblock_db_filename'] = $Javascript->filename;
                $params['wutime_adblock_db_js_key'] =  $Javascript->js_key;
                $params['wutime_adblock_db_js_detect_file'] = $Javascript->js_detect_file;    
            }
        }



        /**
        * ELEMENTS, CSS
        **/
        $params['wutime_adblock'] = self::getUniqId();
        $params['wutime_adblock_full'] = self::getUniqId();
        $params['wutime_adblock_full__box'] = self::getUniqId();
        $params['wutime_adblock_top'] = self::getUniqId();
        $params['wutime_adblock_message'] = self::getUniqId();
        $params['wutime_adblock_close'] = self::getUniqId();
        $params['wutime_notice_content'] = self::getUniqId();


        /**
            JavaScript
            ==================================

            adBlockDetected
            adBlockDetect
            testNode
            runsCounter

            testInterval
            isCheckBlocked
            createNode
            isCheckBlocked

            merge
            defaults
            testNodeClasses
            testNodeStyle

            testInterval
            testRuns
            testValueInterval (40-60)
            testValueRuns (4-12)

        **/
        $params['wutime_adblock_js_adBlockDetected'] = self::getUniqId();
        $params['wutime_adblock_js_adBlockDetector'] = self::getUniqId();
        $params['wutime_adblock_js_testNode'] = self::getUniqId();
        $params['wutime_adblock_js_runsCounter'] = self::getUniqId();

        $params['wutime_adblock_js_testInterval'] = self::getUniqId();
        $params['wutime_adblock_js_isCheckBlocked'] = self::getUniqId();
        $params['wutime_adblock_js_createNode'] = self::getUniqId();
        $params['wutime_adblock_js_isCheckBlocked'] = self::getUniqId();

        $params['wutime_adblock_js_merge'] = self::getUniqId();
        $params['wutime_adblock_js_defaults'] = self::getUniqId();
        $params['wutime_adblock_js_testNodeClasses'] = self::getUniqId();
        $params['wutime_adblock_js_testNodeStyle'] = self::getUniqId();

        $params['wutime_adblock_js_testInterval'] = self::getUniqId();
        $params['wutime_adblock_js_testRuns'] = self::getUniqId();

        $params['wutime_adblock_js_testValueInterval'] = rand(40,60);
        $params['wutime_adblock_js_testValueRuns'] = rand(4,12);

        /**
        * Regular Expression (bot detect)
        **/
        $params['wutime_adblock_js_regex_bots'] = self::getUniqId();
        $params['wutime_adblock_js_regex_bots_val'] = '(googlebot\/|Googlebot-Mobile|Googlebot-Image|Googlebot|Google favicon|Mediapartners-Google|bingbot|slurp|java|wget|curl|Commons-HttpClient|Python-urllib|libwww|httpunit|nutch|phpcrawl|msnbot|jyxobot|FAST-WebCrawler|FAST Enterprise Crawler|biglotron|teoma|convera|seekbot|gigablast|exabot|ngbot|ia_archiver|GingerCrawler|webmon |httrack|webcrawler|grub.org|UsineNouvelleCrawler|antibot|netresearchserver|speedy|fluffy|bibnum.bnf|findlink|msrbot|panscient|yacybot|AISearchBot|IOI|ips-agent|tagoobot|MJ12bot|dotbot|woriobot|yanga|buzzbot|mlbot|yandexbot|purebot|Linguee Bot|Voyager|CyberPatrol|voilabot|baiduspider|citeseerxbot|spbot|twengabot|postrank|turnitinbot|scribdbot|page2rss|sitebot|linkdex|Adidxbot|blekkobot|ezooms|dotbot|Mail.RU_Bot|discobot|heritrix|findthatfile|europarchive.org|NerdByNature.Bot|sistrix crawler|ahrefsbot|Aboundex|domaincrawler|wbsearchbot|summify|ccbot|edisterbot|seznambot|ec2linkfinder|gslfbot|aihitbot|intelium_bot|facebookexternalhit|yeti|RetrevoPageAnalyzer|lb-spider|sogou|lssbot|careerbot|wotbox|wocbot|ichiro|DuckDuckBot|lssrocketcrawler|drupact|webcompanycrawler|acoonbot|openindexspider|gnam gnam spider|web-archive-net.com.bot|backlinkcrawler|coccoc|integromedb|content crawler spider|toplistbot|seokicks-robot|it2media-domain-crawler|ip-web-crawler.com|siteexplorer.info|elisabot|proximic|changedetection|blexbot|arabot|WeSEE:Search|niki-bot|CrystalSemanticsBot|rogerbot|360Spider|psbot|InterfaxScanBot|Lipperhey SEO Service|CC Metadata Scaper|g00g1e.net|GrapeshotCrawler|urlappendbot|brainobot|fr-crawler|binlar|SimpleCrawler|Livelapbot|Twitterbot|cXensebot|smtbot|bnf.fr_bot|A6-Indexer|ADmantX|Facebot|Twitterbot|OrangeBot|memorybot|AdvBot|MegaIndex|SemanticScholarBot|ltx71|nerdybot|xovibot|BUbiNG|Qwantify|archive.org_bot|Applebot|TweetmemeBot|crawler4j|findxbot|SemrushBot|yoozBot|lipperhey|y!j-asr|Domain Re-Animator Bot|AddThis)';
    }

    public static function templater_setup(\XF\Container $container, \XF\Template\Templater &$templater)
    {

        $templater->addFunction('wutime_adblock_explode_newlines', function(\XF\Template\Templater $templater, &$escape, $var1)
        {
           return array_map('trim', explode("\n", $var1));
        });


        $templater->addFunction('wutime_adblock_hex_to_rgb', function(\XF\Template\Templater $templater, &$escape, $var1)
        {
            $hex = $var1;
            list($r, $g, $b) = sscanf($hex, "#%02x%02x%02x");
            $rgb = "$r,$g,$b";
            return $rgb;
        });
    }

    private static function getUniqId() {
        return preg_replace("/[^a-zA-Z0-9]+/", "", uniqid(chr(rand(97,122)),true));
    }
}


