<?php

namespace Wutime\AdBlock;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	public function installStep1()
	{
		$this->schemaManager()->createTable('xf_wutime_adblock', function(Create $table)
		{
			$table->engine('InnoDB');
			$table->addColumn('entity_id', 'int')->primaryKey()->autoIncrement(true,true);
			$table->addColumn('status', 'enum')->values(['Default','Old','New'])->setDefault('New');
			$table->addColumn('path', 'varchar', '255')->setDefault('');
			$table->addColumn('foldername', 'varchar', '60')->setDefault('');
			$table->addColumn('filename', 'varchar', '60')->setDefault('');
			$table->addColumn('js_key', 'varchar', '60')->setDefault('');
			$table->addColumn('js_detect_file', 'varchar', '60')->setDefault('');
			$table->addColumn('created', 'int')->setDefault(0);
			$table->addKey('status');
		});

		$defaults = [
				'status' 			=> 'Default',
				'foldername' 		=> 'wutime_adblock',
				'filename' 			=> 'ads.js',
				'js_key' 			=> 'elaoianb32k',
				'js_detect_file' 	=> 'wutime_detect',
				'created' 			=> \XF::$time,
		];

		\XF::db()->insert('xf_wutime_adblock', $defaults);

		// create first JavaScript
		\Wutime\AdBlock\Cron\CreateJavaScript::install();

	}

    public function upgrade1090771Step1()
	{
		self::installStep1();
    }


	public function uninstallStep1()
	{
		\Wutime\AdBlock\Cron\CreateJavaScript::uninstall();
		$this->schemaManager()->dropTable('xf_wutime_adblock');
	}

}